import json
import traceback

from flask import Flask, request

from execute import execute

app = Flask(__name__)


@app.route('/', methods=['GET'])
def index():
    return app.send_static_file('index.html')


@app.route('/run', methods=['POST'])
def run():
    # noinspection PyBroadException
    try:
        return json.dumps(execute(request.get_data()))
    except:
        return traceback.format_exc(), 500


def main():
    app.run(debug=True)


if __name__ == '__main__':
    main()



from __future__ import print_function

from askso.utils import multiline


class QuietExit(Exception):
    pass


class AccessRecorder(object):
    def __init__(self):
        self.accessed = False

    def __getattr__(self, _):
        self.accessed = True
        raise QuietExit


class FileTracker(object):
    def __init__(self, open_function):
        self.open_function = open_function
        self.paths = []
        self.non_ascii_files = []

    def __call__(self, *args, **kwargs):
        if args and isinstance(args[0], str) and args[0] not in self.paths:
            self.paths.append(args[0])
        return self.open_function(*args, **kwargs)

    def result(self):
        result = []
        for path in self.paths:
            data = self.try_read(path)
            if data is not None:
                result.append({'path': path, 'data': data})
                if len(result) > 3:
                    break
        return result

    def try_read(self, path):
        try:
            with open(path) as f:
                max_lines = 15
                max_bytes = max_lines * 200
                data = f.read(max_bytes + 1).decode('ascii')

            if len(data) > max_bytes or len(data.splitlines()) > max_lines:
                return multiline("""
                    This file is too long.
                    All files opened must be within %s bytes and %s lines at the end of the program.
                    This is so that their contents can easily be included in the question.
                    Cut down your data and code to the minimum you need to show the problem.
                    """ % (max_bytes, max_lines))

            return data

        except IOError:
            pass
        except UnicodeDecodeError:
            self.non_ascii_files.append(path)



from textwrap import dedent


def multiline(message):
    return dedent(message).strip()


def write_to_path(path, text):
    with open(path, 'w') as f:
        f.write(text)


def read_from_path(path):
    with open(path) as f:
        return f.read()



from __future__ import print_function

import os
import platform
import sys
import traceback as tb
try:
    # noinspection PyCompatibility
    from StringIO import StringIO
except ImportError:
    from io import StringIO
from contextlib import contextmanager
from tempfile import gettempdir

from askso.utils import write_to_path
from files import AccessRecorder, QuietExit, FileTracker


@contextmanager
def temp_vars(*var_list):
    holders = []
    try:
        for var in var_list:
            h = Holder()
            h.module, h.name, h.value = var
            h.old = getattr(h.module, h.name)
            setattr(h.module, h.name, h.value)
            holders.append(h)
        yield
    finally:
        for h in holders:
            setattr(h.module, h.name, h.old)


class Holder(object):
    pass


def execute(code):
    if code == 'test_error':
        raise ValueError
    tempdir = gettempdir()
    script_py = os.path.join(tempdir, 'my_script.py')
    write_to_path(script_py, code)
    stdin = AccessRecorder()
    stdout = StringIO()
    stderr = StringIO()
    file_tracker = FileTracker(open)
    with temp_vars([sys, 'stdout', stdout],
                   [sys, 'stderr', stderr],
                   [sys, 'stdin', stdin]):

        # noinspection PyBroadException
        try:
            exec (
                compile(code, script_py, 'exec', dont_inherit=True),
                {'open': file_tracker, '__name__': '__main__'},
                {})
        except QuietExit:
            pass
        except:
            tb_list = [e for e in tb.extract_tb(sys.exc_info()[2]) if os.path.dirname(__file__) not in e[0]]
            print('Traceback (most recent call last):\n' +
                  ''.join(tb.format_list(tb_list) +
                          tb.format_exception_only(*sys.exc_info()[:2])).replace(tempdir, ''),
                  file=sys.stderr)
    return {'stdout': stdout.getvalue(),
            'stderr': stderr.getvalue(),
            'files': file_tracker.result(),
            'stdin_used': stdin.accessed,
            'non_ascii_files': file_tracker.non_ascii_files,
            'version': platform.python_version()}



#!/usr/bin/env python
from codecs import open
from setuptools import setup

with open('README.rst', encoding='utf8') as readme_file:
    readme = readme_file.read()

setup(
    name='ask-so',
    version='1.0.0',
    description='AskSO - StackOverflow Python Question Assistant',
    long_description=readme,
    author='Alex Hall',
    author_email='alex.mojaki@gmail.com',
    license='MIT (Expat)',
    url='https://github.com/alexmojaki/askso',
    packages=['askso'],
    entry_points={
        'console_scripts': ['askso=askso.server:main'],
    },
    install_requires=[
        'Flask >= 0.10.0, < 1.0.0',
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
